@@ -1,6 +1,10 @@
+from __future__ import unicode_literals
+from django.contrib.auth.models import Group
 from django.db import models
+from django.utils.encoding import python_2_unicode_compatible
 
-from mptt.models import MPTTModel
+import mptt
+from mptt.models import MPTTModel, TreeForeignKey
 from mptt.managers import TreeManager
 
 
@@ -8,22 +12,24 @@ class CustomTreeManager(TreeManager):
     pass
 
 
+@python_2_unicode_compatible
 class Category(MPTTModel):
     name = models.CharField(max_length=50)
     parent = models.ForeignKey('self', null=True, blank=True, related_name='children')
 
-    def __unicode__(self):
+    def __str__(self):
         return self.name
 
     def delete(self):
         super(Category, self).delete()
 
 
+@python_2_unicode_compatible
 class Genre(MPTTModel):
     name = models.CharField(max_length=50, unique=True)
     parent = models.ForeignKey('self', null=True, blank=True, related_name='children')
 
-    def __unicode__(self):
+    def __str__(self):
         return self.name
 
 
@@ -31,6 +37,7 @@ class Insert(MPTTModel):
     parent = models.ForeignKey('self', null=True, blank=True, related_name='children')
 
 
+@python_2_unicode_compatible
 class MultiOrder(MPTTModel):
     name = models.CharField(max_length=50)
     size = models.PositiveIntegerField()
@@ -38,9 +45,9 @@ class MultiOrder(MPTTModel):
     parent = models.ForeignKey('self', null=True, blank=True, related_name='children')
 
     class MPTTMeta:
-        order_insertion_by = ['name', 'size', 'date']
+        order_insertion_by = ['name', 'size', '-date']
 
-    def __unicode__(self):
+    def __str__(self):
         return self.name
 
 
@@ -54,6 +61,7 @@ class Node(MPTTModel):
         tree_id_attr = 'work'
 
 
+@python_2_unicode_compatible
 class OrderedInsertion(MPTTModel):
     name = models.CharField(max_length=50)
     parent = models.ForeignKey('self', null=True, blank=True, related_name='children')
@@ -61,7 +69,7 @@ class OrderedInsertion(MPTTModel):
     class MPTTMeta:
         order_insertion_by = ['name']
 
-    def __unicode__(self):
+    def __str__(self):
         return self.name
 
 
@@ -76,6 +84,7 @@ class NewStyleMPTTMeta(MPTTModel):
         left_attr = 'testing'
 
 
+@python_2_unicode_compatible
 class Person(MPTTModel):
     name = models.CharField(max_length=50)
     parent = models.ForeignKey('self', null=True, blank=True, related_name='children')
@@ -84,10 +93,7 @@ class Person(MPTTModel):
     objects = models.Manager()
     my_tree_manager = CustomTreeManager()
 
-    class MPTTMeta:
-        tree_manager_attr = 'my_tree_manager'
-
-    def __unicode__(self):
+    def __str__(self):
         return self.name
 
 
@@ -95,6 +101,17 @@ class Student(Person):
     type = models.CharField(max_length=50)
 
 
+@python_2_unicode_compatible
+class CustomPKName(MPTTModel):
+    my_id = models.AutoField(db_column='my_custom_name', primary_key=True)
+    name = models.CharField(max_length=50)
+    parent = models.ForeignKey('self', null=True, blank=True,
+            related_name='children', db_column="my_cusom_parent")
+
+    def __str__(self):
+        return self.name
+
+
 # for testing various types of inheritance:
 
 # 1. multi-table inheritance, with mptt fields on base class.
@@ -159,11 +176,14 @@ class DoubleProxyModel(SingleProxyModel):
         proxy = True
 
 
-class CustomPKName(MPTTModel):
-    my_id = models.AutoField(db_column='my_custom_name', primary_key=True)
-    name = models.CharField(max_length=50)
-    parent = models.ForeignKey('self', null=True, blank=True,
-            related_name='children', db_column="my_cusom_parent")
+class AutoNowDateFieldModel(MPTTModel):
+    parent = models.ForeignKey('self', null=True, blank=True, related_name='children')
+    now = models.DateTimeField(auto_now_add=True)
 
-    def __unicode__(self):
-        return self.name
+    class MPTTMeta:
+        order_insertion_by = ('now',)
+
+
+# test registering of remote model
+TreeForeignKey(Group, blank=True, null=True).contribute_to_class(Group, 'parent')
+mptt.register(Group, order_insertion_by=('name',))
